#-*- coding: utf-8 -*-

import argparse
import sys
import os

try:
    from l2lss.dcvsapi import Repo
    from l2lss.init import get_empty_config
    from l2lss.pull import sync_repo
    from l2lss.utils import concat_path
except ImportError:
    from dcvsapi import Repo
    from init import get_empty_config
    from pull import sync_repo
    from utils import concat_path


def clone(*sys_args):
    """Clone a l2lss repo."""
    # Parse ags
    args = parse(*sys_args)
    check(args)

    # Get the code
    clone_repo(args.dcvs_url, args.clone_path, args.git, args.hg)

    # Get the files
    l2lss_folder = concat_path(args.clone_path, '.l2lss')
    os.chdir(args.clone_path)
    os.makedirs(l2lss_folder)
    config = get_empty_config()
    config['path'] = {'sync path': args.path}
    sync_repo(l2lss_folder, config)


def parse(*sys_args):
    """Parse the args from sys.args and returns the result."""
    parser = argparse.ArgumentParser(description='''Clone a local2local_repo. If no repository type is defined, we will use git.
    All commands are run where the script is.''')
    parser.add_argument('path',
                            help='The path where the repo to clone is located.')
    parser.add_argument('--clone-path',  '-p',  dest='clone_path',  default='.',
                            help='The path were to clone the local2local_repo.')
    parser.add_argument('--dcvs-url', '-u', dest='dcvs_url',
                            help='Where to clone the dcvs code')
    parser.add_argument('--git', dest='git', action='store_true',
                            help='Use a git repo for code.')
    parser.add_argument('--hg',  '--mercurial',  dest='hg',  action='store_true',
                            help='Use a mercurial repo for code.')
    args = parser.parse_args()

    return args


def check(args):
    """Exit the script if it identifies incoherences."""
    if args.git and args.hg:
        print('Please choose your dcvs.')
        sys.exit(1)


def clone_repo(url, path, git, hg):
    repo_type = None
    if git:
        repo_type = 'git'
    elif hg:
        repo_type = 'hg'
    if repo_type:
        Repo.clone(url, path, type=repo_type)
