#-*- coding: utf-8 -*-

import argparse
import sys

try:
    from l2lss.utils import check_is_in_l2lss_repo, get_config_dir, get_config_file, get_config_parser
except ImportError:
    from utils import check_is_in_l2lss_repo, get_config_dir, get_config_file, get_config_parser

def config(*sys_args):
    args = parse(*sys_args)
    check_is_in_l2lss_repo()

    # If no argument is provided, print the config file.
    if not args.config_identifier:
        display_config_file()
        sys.exit(0)

    # If len(args.config_identifier) == 2, set the config.
    # Else, print it.
    config = get_config_parser()
    try:
        section, key = args.config_identifier[0].split('.')
    except ValueError:
        print('Section identifier is incorrect.')
        sys.exit(1)

    if len(args.config_identifier) == 2:
        try:
            config[section][key] = args.config_identifier[1]
            with open(get_config_file(), 'w') as config_file:
                config.write(config_file)
        except KeyError:
            print('Section identifier is incorrect.')
            sys.exit(1)
    else:
        try:
            print(config[section][key])
        except KeyError:
            print('Section identifier is incorrect.')
            sys.exit(1)


def parse(*sys_args):
    """Parse the args from sys.args and returns the result."""
    parser = argparse.ArgumentParser(description='''Print or modify l2lss configuration.''')
    parser.add_argument('config_identifier', nargs='*', type=str,
                            help='Set the config_identifier to the provided value. Print current value if no value is provided.')
    args = parser.parse_args()

    return args


def display_config_file():
    """Display the config file."""
    config_file = get_config_file()
    with open(config_file, 'r') as config:
        print(config.read())
