# -*- coding: utf-8 -*-
"""
    Python API to Mercurial and Git using the command-line interface.
"""
import gitapi
import hgapi

class Repo:
    """A representation of a Mercurial or Git repository."""

    def __init__(self,  path='.',  user=None,  type=None):
        if type.lower() == 'mercurial':
            type = 'hg'
        self.type = type
        if self.type == 'git':
            self.repo = gitapi.Repo(path,  user=user)
        elif self.type == 'hg':
            self.repo = hgapi.Repo(path,  user=user)

    def init(self):
        """Init the repo"""
        getattr(self.repo, self.type + '_init')()

    def fetch(self, source=None):
        """Fetch changes to this repo (git fetch, hg pull)."""
        if self.type == 'git':
            self.repo.git_fetch(source)
        elif self.type == 'hg':
            self.repo.hg_pull(source)

    def pull(self, source=None):
        """Pull changes as git does, ie fetch + merge.
        For mercurial running hg pull && hg update
        """
        getattr(self.repo,  self.type + '_pull')(source)
        if self.type == 'hg':
            self.repo.hg_command('update')

    def push(self, destination=None):
        """Push changes of this repo."""
        getattr(self.repo,  self.type + '_push')(destination)

    def update(self, reference, clean=False):
        """Update the working copy of this repo to reference (git co reference, hg update -r reference)."""
        if self.type == 'git':
            self.repo.git_checkout(reference, clean)
        elif self.type == 'hg':
            self.repo.hg_update(reference, clean)

    def checkout(self, reference, clean=False):
        """Update the working copy of this repo to reference (git co reference, hg update -r reference)."""
        self.update(reference, clean)

    def add(self, filepath):
        """Add filepath to the repo."""
        getattr(self.repo, self.type + '_add')(filepath)

    def commit(self, message, user=None, files=[], close_branch=False):
        """Commit changes to the repository."""
        getattr(self.repo, self.type + '_commit')(message,  user=user, files=files, close_branch=close_branch)

    @classmethod
    def clone(cls,  url,  path, type='git', *arg):
        if type == 'git':
            gitapi.Repo.git_clone(url,  path,  *arg)
        elif type == 'hg':
            hgapi.Repo.hg_clone(url,  path,  *arg)
