#-*- coding: utf-8 -*-
import os, shutil
from sh import python3
from l2lss.utils import is_l2lss_init, concat_path


launch_path = os.getcwd()
main_path = concat_path(launch_path, 'l2lss/main.py')
repo_test_path = concat_path(os.getcwd(), '/test/test_repo')
repo_server_path = concat_path(os.getcwd(), '/test/test_server')
repo_test_pull_path = concat_path(os.getcwd(),  '/test/test_pull')
repo_test_clone_path = concat_path(os.getcwd(),  '/test/test_clone')


def create_test_dir():
    if os.path.exists(repo_test_path):
        shutil.rmtree(repo_test_path)
    os.makedirs(repo_test_path)


def setup_module(module):
    # Create test dir
    python3(main_path, 'init', p=repo_test_path, git=True, s=repo_server_path,
                r='drush sql-cli', f='sites/default')
    os.makedirs(repo_test_path + '/sites/default')
    python3(main_path, 'init', p=repo_test_pull_path, n=True, s=repo_server_path, f='sites/default')
    os.makedirs(repo_test_pull_path + '/sites/default')
    os.mkdir(repo_server_path)


def teardown_module(module):
    if os.path.exists(repo_test_path):
        shutil.rmtree(repo_test_path)
    if os.path.exists(repo_server_path):
        shutil.rmtree(repo_server_path)
    if os.path.exists(repo_test_pull_path):
        shutil.rmtree(repo_test_pull_path)
    if os.path.exists(repo_test_clone_path):
        shutil.rmtree(repo_test_clone_path)


def test_init():
    """Check that the git repo and l2l-ss are initialized."""
    path_git = concat_path(repo_test_path, '.git')
    assert os.path.exists(path_git)
    path_l2lss = concat_path(repo_test_path, '.l2lss')
    assert os.path.exists(path_l2lss)

    # Check that the config file exits
    path_config_file = concat_path(repo_test_path, '.l2lss/config')
    assert os.path.exists(path_config_file)

    # Check that the content of the config file is correct.
    with open(path_config_file,  'r', encoding='utf-8') as config_file:
        config = config_file.read()
        true_config = '\n'.join(['[path]', 'sync path = {}'.format(repo_server_path), '',
                                    '[database]',
                                    'restore cmd = drush sql-cli', '',
                                    '[files]',
                                    'files dir = sites/default', '',  ''])
        assert config == true_config


def test_is_init():
    """Check that the is_l2lss_init function detects that the repo is setup."""
    os.chdir(repo_test_path)
    assert is_l2lss_init()
    os.chdir('.l2lss')
    assert is_l2lss_init()
    os.chdir(launch_path)


def test_push():
    """Tests that the push command works as intended.

    No database operations are perform to avoid making any assumption on the testing configuration.
    """
    # It must be launched with the l2lss repo.
    os.chdir(repo_test_path)
    # Add a file in the file directory
    open('sites/default/settings.py', 'a').close()
    # Push
    python3(main_path, 'push')
    files = os.listdir(repo_server_path)
    # Check that we have config and the sync_folder
    assert len(files) == 2
    assert 'config' in files
    files.remove('config')
    repo_server_files = concat_path(repo_server_path, files[0])
    assert os.listdir(repo_server_files) == ['default']
    assert os.listdir(concat_path(repo_server_files, 'default')) == ['settings.py']
    os.chdir(launch_path)


def test_pull():
    """Tests that the pull command works as intended.

    No database operations are perform to avoid making any assumption on the testing configuration.
    """
    # It must be launched with the l2lss repo.
    os.chdir(repo_test_pull_path)
    python3(main_path, 'pull')
    files = os.listdir(concat_path(repo_test_pull_path, '.l2lss'))
    # Check that we have config and the files folder in the sync_folder
    assert len(files) == 2
    assert 'config' in files
    files.remove('config')
    repo_test_pull_files = concat_path(repo_test_pull_path, '.l2lss', files[0])
    assert os.listdir(repo_test_pull_files) == ['default']
    assert os.listdir(concat_path(repo_test_pull_files, 'default')) == ['settings.py']
    # Check that the files are correctly updated
    assert os.path.exists(concat_path(repo_test_pull_path, 'sites/default/settings.py'))
    os.chdir(launch_path)


def test_clone():
    """Test that the clone command works as intended."""
    python3(main_path, 'clone', repo_server_path, p=repo_test_clone_path, u=repo_test_path, git=True)

    # Test that we have .git and .l2lss
    path_git = concat_path(repo_test_clone_path, '.git')
    assert os.path.exists(path_git)
    path_l2lss = concat_path(repo_test_clone_path, '.l2lss')
    assert os.path.exists(path_l2lss)
    return
    # Check that the config file exits
    path_config_file = concat_path(repo_test_clone_path, '.l2lss/config')
    assert os.path.exists(path_config_file)

    # Check that the content of the config file is correct.
    with open(path_config_file,  'r', encoding='utf-8') as config_file:
        config = config_file.read()
        true_config = '\n'.join(['[path]', 'sync path = {}'.format(repo_server_path), '',
                                    '[database]',
                                    'restore cmd = drush sql-cli', '',
                                    '[files]',
                                    'files dir = sites/default', '',  ''])
        assert config == true_config

    # Check that the files are OK.
    assert os.path.exits(concat_path(repo_test_clone_path, 'sites/default/settings.py'))
